@@ -1,15 +1,172 @@
 from datetime import datetime
+import this
 import numpy as np
 from .utils.io_container_data import IOContainerData
 from .banco_dados.conexao_banco import ConexaoBanco
 import pandas as pd
 import os
 import logging
-from multiprocessing.pool import ThreadPool
+import threading
 
-class Start():
 
-    def __init__(self, path_file, sep=',',  header=0, dtype=None) :
+class Util():
+
+    def get_path_file_bronze(path_file, sub_dir, is_date=True):
+        return Util.get_path_file(path_file, 'incoming', sub_dir, is_date)
+
+
+    def get_path_file_silver(path_file, sub_dir, is_date=True):
+        return Util.get_path_file(path_file, 'input', sub_dir, is_date)
+
+    def get_path_file(path_file, sub_dir_origem , sub_dir_destino, is_date=True):
+        path = os.path.dirname(path_file).replace(sub_dir_origem, sub_dir_destino)
+        if(is_date):
+            return path + str( datetime.today().strftime('%Y-%m-%d') )
+        else:
+            return path
+
+    def get_name_file(path_file,is_hora=True):
+
+        name = os.path.basename(path_file)
+        if(is_hora):
+            return str( datetime.today().strftime('%H.%M.%S_') ) + name
+        else:
+            return name
+
+    def get_chunksize():
+        return 100000
+
+
+class Bronze():
+
+    def __init__(self, path_file, sep=',',  header=0, dtype=None):
+    
+        self.ioContainerData = IOContainerData()
+        self.path_file = path_file
+        self.sep = sep
+        self.sep_processado = ';'
+        self.header = header
+        self.dtype = dtype
+
+
+    def start(self):
+
+        msg = ""
+        sucesso = True
+
+        try:
+            logging.info(f"Arquivo {self.path_file}")
+            self.__break_file()
+            self.__backup()
+        except Exception as e:
+            logging.error(f"Falha no processo {e}")
+            msg = msg +  f"\nFalha no processo {e}. "
+            sucesso = False
+        finally:
+            logging.info("Fim do processo")
+            msg = msg + "\nFim do processo. "
+
+        return sucesso, msg, self.asynchronous
+
+
+    def __break_file(self):
+
+        df_data = self.ioContainerData.load(
+            self.path_file, 
+            self.ioContainerData.name_container_bronze, 
+            sep=self.sep, 
+            header=self.header,  
+            dtype=self.dtype,
+            chunksize = Util.get_chunksize()
+        )
+
+        file_name = Util.get_name_file(self.path_file, 
+                                        is_hora=False)
+
+        path_input_enrichment_input = Util.get_path_file_bronze(self.path_file,
+                                        self.ioContainerData.dir_input_enrichment_input,
+                                        is_date=False)
+
+        
+        logging.info(f"Nome do arquivo {file_name}")  
+        logging.info(f"Caminho do arquivo {path_input_enrichment_input}")
+        logging.info(f"Quantidade de lote(s) {len(df_data)}")
+
+        self.__delele_file(path_input_enrichment_input)
+
+        count = 1
+        threads = list()
+
+        for chunk in df_data:
+    
+            logging.info(f"Iniciando quebra do arquivo {self.path_file} do lote {count}")
+
+            if isinstance(chunk, pd.DataFrame):
+                _df_data = chunk
+            else:
+                _df_data = next(chunk)
+
+            file_input_enrichment_input = os.path.join(path_input_enrichment_input, file_name + "_"+str(count)+".csv")
+
+            t = threading.Thread(target=self.ioContainerData.download, 
+                        args=(_df_data,
+                            file_input_enrichment_input,
+                            self.ioContainerData.name_container_silver,
+                            self.sep_processado)
+                        )
+            t.start()
+            threads.append(t)
+
+            del(_df_data)
+            logging.info(f"Fim da quebra do arquivo {self.path_file} do lote {count}")
+            count = count + 1
+
+
+        for thread in threads:
+            thread.join()
+
+
+
+    def __delele_file(self, dir):
+        
+        self.ioContainerData.delete_to_blob(
+            self.ioContainerData.name_container_silver,
+            dir)
+
+    def __backup(self):
+    
+        file_name = Util.get_name_file(self.path_file)
+        path_backup = Util.get_path_file_bronze(self.path_file, self.ioContainerData.dir_backup)
+        backup = os.path.join(path_backup, file_name)
+
+        self.ioContainerData.cp_file( self.path_file, 
+                                      backup, 
+                                      self.ioContainerData.name_container_bronze
+                                    )
+        logging.info(f'Arquivo salvo em backup {self.ioContainerData.name_container_bronze}/{backup}')
+
+    #def __copy_input_enrichment(self):
+    
+    #    file_name = __get_name_file(is_hora=False)
+    #    path_input_enrichment_input =  __get_path_file(self.ioContainerData.dir_input_enrichment_input,is_date=False)
+    #    file_input_enrichment_input = os.path.join(path_input_enrichment_input, file_name)
+
+    #    self.ioContainerData.cp_file( 
+    #        self.path_file, 
+    #        file_input_enrichment_input, 
+    #        self.ioContainerData.name_container_bronze,
+    #        self.ioContainerData.name_container_silver )
+  
+    #    logging.info(f'Arquivo movido para {file_input_enrichment_input}')
+
+    #    return file_input_enrichment_input
+
+
+
+""" Classe do silver"""
+class Silver():
+
+    def __init__(self, path_file, sep=';',  header=0, dtype=None) :
 
         self.ioContainerData = IOContainerData()
         self.conexaoBanco = ConexaoBanco()
@@ -17,16 +174,8 @@ class Start():
         self.sep = sep
         self.header = header
         self.dtype = dtype
-        self.chunksize=10000
 
-    def start(self):
-
-        logging.info(f"Arquivo {self.path_file}")
-
-        self.file_input_enrichment = self.__copy_input_enrichment()
-
-
-    def enrichment(self, dictParametros ):
+    def run(self, dictParametros ):
 
         logging.info(f"Iniciando o processa de tratamento do arquivo")
 
@@ -34,62 +183,47 @@ class Start():
         logging.info(f"Posição do cabeçalho {self.header}")
         logging.info(f"Tipo de dado {self.dtype}")
 
-        df_data = self.ioContainerData.load(
-                                self.file_input_enrichment, 
-                                self.ioContainerData.name_container_silver, 
-                                sep=self.sep, 
-                                header=self.header,  
-                                dtype=self.dtype,
-                                chunksize=self.chunksize
-                            )
+        df_datas = self.ioContainerData.load(
+            self.path_file, 
+            self.ioContainerData.name_container_silver, 
+            sep=self.sep, 
+            header=self.header,  
+            dtype=self.dtype,
+            chunksize= Util.get_chunksize()
+        )
 
-
-        extensao = self.ioContainerData.get_extension(self.file_input_enrichment)
-
-        logging.info(f"Extensão do arquivo {extensao}")
-
-        if extensao == ".xlsx" or extensao == ".xls":
+        _is_first_file = self.__is_first_file()
+        count = 1
+        for df_data in df_datas:
             try:
-                logging.info(f"Iniciando o processo simples")
-                self.dados_original = df_data
-                self.__processamento(dictParametros, True)
+            
+                logging.info(f"Iniciando o processo de enrichment {count}")
+                if isinstance(df_data, pd.DataFrame):
+                     self.dados_original = df_data
+                else:
+                    self.dados_original = next(df_data)
+
+                self.__processamento(dictParametros, _is_first_file and (count == 1) )
+
+                count = count + 1 
                 del(self.dados_original)
-                logging.info(f"Final o processo simples\n")
+                logging.info(f"Fim do processo de enrichment {count}")
                 
             except Exception as error:
                 raise Exception(f"{error}")
 
-        else:
 
-            pool = ThreadPool(processes=4)
+        self.__delele_file()
 
-            count = 1
-            for chunk in df_data:
+
+    def __delele_file(self):
+        self.ioContainerData.delete_to_blob(
+            self.ioContainerData.name_container_silver,
+            self.path_file)
         
-                try:
-                    logging.info(f"Iniciando o processo do lote {count}")
 
-                    if isinstance(chunk, pd.DataFrame)  :
-                        self.dados_original = chunk
-                    else:
-                        self.dados_original = next(chunk)
-
-                    #self.__processamento(dictParametros, (count == 1))
-                    async_result = pool.apply_async( self.__processamento(dictParametros, (count == 1)) )
-                    
-                    del(self.dados_original)
-
-                    del(chunk)
-
-                    logging.info(f"Final o processo do lote {count}\n")
-
-                    count = count + 1    
-                except Exception as error:
-                    raise Exception(f"{error}")
-
-        del(df_data)
-
-        logging.info("Fim do processo de enrichment")
+    def __is_first_file(self):
+        return self.path_file.endswith("_1.csv")
 
 
     def __processamento(self, dictParametros, is_first_processing):
@@ -106,16 +240,14 @@ class Start():
                 self.dados = self.__get_field(parametro.list_columns_to_use)
 
                 self.__rename_field(parametro.list_columns_rename)
-
+               
                 self.__enrichment_sql(parametro.list_parametros_enrichment_sql)
-
-                self.__delete_lines(parametro.list_lines_delete)
-            
-                self.__drop_lines_duplicates(parametro.list_lines_duplicates, key, is_first_processing)
-
+               
                 self.__delete_columns(parametro.list_columns_delete)
-
-                #self.__delete_lines(parametro.list_lines_delete)
+              
+                self.__delete_lines(parametro.list_lines_delete)
+               
+                self.__drop_lines_duplicates(parametro.list_lines_duplicates, key, is_first_processing)
 
                 self.__format_date(parametro.list_columns_format_date)
 
@@ -124,7 +256,7 @@ class Start():
                 self.__reorganize_field(parametro.list_field_reorganize)
 
                 self.__save_enrichment()
-
+       
                 self.__insert_data_base(key, parametro.type_insert_table , is_first_processing)
 
                 del(self.dados)
@@ -132,43 +264,16 @@ class Start():
         except Exception as error:
             raise Exception(f"{error}")
 
-    def __copy_input_enrichment(self):
-
-        file_name = self.__get_name_file(is_hora=False)
-        path_input_enrichment =  self.__get_path_file(self.ioContainerData.dir_input_enrichment,is_date=False)
-        file_input_enrichment = os.path.join(path_input_enrichment, file_name)
-
-        self.ioContainerData.cp_file( 
-            self.path_file, 
-            file_input_enrichment, 
-            self.ioContainerData.name_container_bronze,
-            self.ioContainerData.name_container_silver )
-  
-        logging.info(f'Arquivo movido para {file_input_enrichment}')
-
-        return file_input_enrichment
-
-    def backup(self):
-
-        file_name = self.__get_name_file()
-        path_backup = self.__get_path_file(self.ioContainerData.dir_backup)
-        backup = os.path.join(path_backup, file_name)
-
-        self.ioContainerData.mv_file( self.file_input_enrichment, 
-                                      backup, 
-                                      self.ioContainerData.name_container_silver, 
-                                      self.ioContainerData.name_container_bronze 
-                                    )
-        logging.info(f'Arquivo salvo em backup {self.ioContainerData.name_container_bronze}/{backup}')
 
+        logging.info(f'Arquivo removido')
 
     def fail(self):
    
-        file_name = self.__get_name_file()
-        path_failed = self.__get_path_file( self.ioContainerData.dir_failed)
+        file_name = Util.get_name_file(self.path_file)
+        path_failed = Util.get_path_file_bronze(self.path_file, self.ioContainerData.dir_failed)
         failed = os.path.join(path_failed, file_name)
 
-        self.ioContainerData.mv_file( self.file_input_enrichment, 
+        self.ioContainerData.mv_file( self.path_file, 
                                       failed, 
                                       self.ioContainerData.name_container_silver, 
                                       self.ioContainerData.name_container_bronze 
@@ -225,7 +330,7 @@ class Start():
                     filtros = []
                     for lines_duplicates in list_lines_duplicates:
                         #df[df['column'].str.lower().isin([x.lower() for x in mylist])]
-                        filtros.append( ~self.dados[lines_duplicates].str.lower().isin( [x.lower() for x in _pd[lines_duplicates]] ) )
+                        filtros.append( ~self.dados[lines_duplicates].isin( _pd[lines_duplicates] ) )
 
                         #filtros.append( ~self.dados[lines_duplicates].str.lower().isin(_pd[lines_duplicates].str.upper() ) )
 
@@ -281,6 +386,7 @@ class Start():
         except Exception as error:
             raise Exception(f"Erro ao reorganizar campos {error}")
 
+
     #list_column_formt_date = ('Data de Criação Op','%d/%m/%Y')
     def __format_date(self,list_column_formt_date):
 
@@ -299,6 +405,7 @@ class Start():
             raise Exception(f"Erro ao formatar data {error}")
 
 
+
     #list_analytics = [('teste','"joao" if( x["TIPO"].upper() == "MATRIZ" ) else None')]
     def __create_columns_analytics_apply(self,list_analytics):
         
@@ -317,17 +424,17 @@ class Start():
     def __save_enrichment(self):
  
         try:
-            file_name = self.__get_name_file()
-            path_enrichment = self.__get_path_file('enrichment', is_date=False)
+            file_name = Util.get_name_file(self.path_file)
+            path_enrichment = Util.get_path_file_silver(self.path_file, 'enrichment', is_date=False)
 
             path_out_file_enrichment = os.path.join(path_enrichment, file_name)
 
             self.ioContainerData.upload_frame_to_blob( 
                 self.ioContainerData.name_container_silver, 
                 path_out_file_enrichment, 
-                self.dados)
+                self.dados, 'parquet')
 
-            logging.info(f'Arquivo movido para {path_out_file_enrichment}')
+            logging.info(f'Arquivo enriquecido salvo em {path_out_file_enrichment}')
             
         except Exception as error:
             raise Exception(f"Erro ao salvar dados enriquecido {error}")
@@ -400,16 +507,6 @@ class Start():
             raise Exception(f"Erro no enriquecimento por sql{error}")
 
 
-    def __get_name_file(self,is_hora=True):
-
-        name = os.path.basename(self.path_file)
-        if(is_hora):
-            return str( datetime.today().strftime('%H.%M.%S_') ) + name
-        else:
-            return name
-
-
-
     def __rename_field(self, list_columns_rename):
 
         try:
@@ -420,35 +517,29 @@ class Start():
             raise Exception(f"Erro ao renomear os campos {error}")
 
 
-    def __get_path_file(self, sub_dir, is_date=True):
-        path = os.path.dirname(self.path_file).replace('incoming', sub_dir)
-        if(is_date):
-            return path + str( datetime.today().strftime('%Y-%m-%d') )
-        else:
-            return path
 
             
-
-    def run(self, myDict)->str:
+    def start(self, myDict)->str:
 
         msg = ""
+        sucesso = True
         
         try:
-            self.start()   
-            self.enrichment(myDict)
-            self.backup()
+            self.run(myDict)
             logging.info(f"Processamento finalizado com sucesso arquivo {self.path_file}")
             msg = msg + f"\nProcessamento finalizado com sucesso arquivo {self.path_file}. "
+            sucesso = True
         except Exception as e:
             logging.error(f"Falha no processo {e}")
             msg = msg +  f"\nFalha no processo {e}. "
+            sucesso = False
             self.fail()
         finally:
             self.__close_base()
             logging.info("Fim do processo")
             msg = msg + "\nFim do processo. "
 
-        return msg
+        return sucesso, msg
 
 
     
\ No newline at end of file
